@@ -1,7 +1,7 @@
 """
 pylibftdi - python wrapper for libftdi
 
-Copyright (c) 2010 Ben Bass <benbass@codedstructure.net>
+Copyright (c) 2010-2011 Ben Bass <benbass@codedstructure.net>
 See LICENSE file for details and (absence of) warranty
 
 pylibftdi: http://bitbucket.org/codedstructure/pylibftdi
@@ -20,10 +20,20 @@ class Driver(object):
     This is where it all happens...
     We load the libftdi library, and use it.
     """
-    def __init__(self):
+    def __init__(self, mode="b", encoding="latin1"):
         self.ctx = None
         self.fdll = ParrotEgg()
         self.opened = False
+        # mode can be either 'b' for binary, or 't' for text.
+        # if set to text, the values returned from read() will
+        # be decoded using encoding before being returned as
+        # strings; for binary the raw bytes will be returned.
+        # This will only affect Python3.
+        self.mode = mode
+        # when giving a str to Driver.write(), it is encoded.
+        # default is latin1, because it provides
+        # a one-to-one correspondence for code points 0-FF
+        self.encoding = encoding
         # ftdi_usb_open_dev initialises the device baudrate
         # to 9600, which certainly seems to be a de-facto
         # standard for serial devices.
@@ -32,7 +42,7 @@ class Driver(object):
     def open(self):
         "open connection to a FTDI device"
         if self.opened:
-            return self
+            return
         # TODO: provide parameter to select required device
         # (if multiple are attached)
         ftdi_lib = find_library('ftdi')
@@ -60,7 +70,6 @@ class Driver(object):
         #   (except we don't have deterministic destructors. oh well.)
         self.fdll = fdll
         self.opened = True
-        return self
 
     def close(self):
         "close our connection, free resources"
@@ -69,6 +78,7 @@ class Driver(object):
             self.fdll.ftdi_deinit(byref(self.ctx))
         self.fdll = DeadParrot()
 
+
     @property
     def baudrate(self):
         """
@@ -76,33 +86,50 @@ class Driver(object):
         to ensure baudrate was accepted by the driver.
         """
         return self._baudrate
+
     @baudrate.setter
     def baudrate(self, value):
         result = self.fdll.ftdi_set_baudrate(byref(self.ctx), value)
         if result == 0:
             self._baudrate = value
 
+
     def read(self, length):
-        "read a string of upto length bytes from the FTDI device"
+        """
+        read upto length bytes from the FTDI device
+        return type depends on self.mode - if 'b' return
+        raw bytes, else decode according to self.encoding
+        """
         buf = create_string_buffer(length)
         rlen = self.fdll.ftdi_read_data(byref(self.ctx), byref(buf), length)
         if rlen == -1:
             raise FtdiError(self.get_error_string())
-        return buf.raw[:rlen]
+        byte_data = buf.raw[:rlen]
+        if self.mode == 'b':
+            return byte_data
+        else:
+            return byte_data.decode(self.encoding)
 
     def write(self, data):
         "write given data string to the FTDI device"
-        buf = create_string_buffer(data)
+        try:
+            byte_data = bytes(data)
+        except TypeError:
+            # this will happen if we are Python3 and data is a str.
+            byte_data = data.encode(self.encoding)
+        buf = create_string_buffer(byte_data)
         written = self.fdll.ftdi_write_data(byref(self.ctx),
                                             byref(buf), len(data))
         if written == -1:
             raise FtdiError(self.get_error_string())
         return written
 
+
     def get_error_string(self):
         "return error string from libftdi driver"
         return self.fdll.ftdi_get_error_string(byref(self.ctx))
 
+
     @property
     def ftdi_fn(self):
         """
@@ -110,7 +137,7 @@ class Driver(object):
         which are called with a pointer to a ftdi_context
         struct as the first parameter to be called here
         in a nicely encapsulated way:
-        >>> with FtdiDriver() as drv:
+        >>> with Driver() as drv:
         >>>     # set 8 bit data, 2 stop bits, no parity
         >>>     drv.ftdi_fn.ftdi_set_line_property(8, 2, 0)
         >>>     ...
@@ -124,9 +151,19 @@ class Driver(object):
                                           byref(self.ctx))
         return FtdiForwarder()
 
+
     def __enter__(self):
-        "support for context manager"
-        return self.open()
+        """
+        support for context manager.
+        Note the driver is opened and closed automatically
+        when used in a with statement, and the driver object
+        itself is returned:
+        >>> with Driver(mode='t') as drv:
+        >>>     drv.write('Hello World!')
+        >>>
+        """
+        self.open()
+        return self
 
     def __exit__(self, exc_type, exc_val, tb):
         "support for context manager"
